
/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : configuration.cpp
**      SYSTEM NAME    : ACL - Andromeda Class Library
**      VERSION NUMBER : $Revision: 1.6 $
**
**  DESCRIPTION      :  Implementation of configuration class
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Jul 23, 2002
**      LAST UPDATE     : Apr 13, 2007
**      MODIFICATIONS   : 
**************************************************************************/

#include <fcntl.h>
#include <unistd.h>
#include "configuration.h"


/*=========================================================================
**  NAME           : configuration::read
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : true if any config files were found
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Nov 22, 2002
**=========================================================================
*/

bool configuration::read(const String name)
{
   String     filename;

   app_name   = name;

   //   Try to read the config files.
   //   Two configuarions are read. The system and the user config.
   //   First system config in /etc and if it does not exist, in /usr/local/etc
 
   filename = "/etc/" + name + ".conf";
   //  Check if we can read the file.

   int fd = open(filename, O_RDONLY);
   if (fd < 0)
   {
      filename = "/usr/local/etc/" + name + ".conf";
      fd = open(filename, O_RDONLY);
   }

   if (fd > 0)
   {
      close(fd);
      system = xmlParseFile(filename);
   }

   //  The user config is read from the current directory.
   //  If that does not exist, try the home directory.

   filename = name + ".conf";
   fd = open(filename, O_RDONLY);
   if (fd < 0)
   {
      if (getenv("HOME") != NULL)
      {
         filename = getenv("HOME");
         filename += "/.";
         filename += name + ".conf";
         fd = open(filename, O_RDONLY);
      }
   }
   if (fd > 0)
   {
      close(fd);
      user = xmlParseFile(filename);
   }

   //  Check the root element, which must be the application name

   xmlNodePtr   root;

   if (system != NULL)
   {
      root = xmlDocGetRootElement(system);
      if (app_name != (const char *)root->name)
      {
         std::cerr << "Wrong config file.\n";
         xmlFreeDoc(system);
         system = NULL;
      }
   }

   if (user != NULL)
   {
      root = xmlDocGetRootElement(user);
      if (app_name != (const char *)root->name)
      {
         std::cerr << "Wrong config file.\n";
         xmlFreeDoc(user);
         user = NULL;
      }
   }

   return system != NULL || user != NULL;
}

/*=========================================================================
**  NAME           : configuration::xmlFindTag
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : Pointer to the XML element or NULL
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Apr 13, 2007
**=========================================================================
*/

xmlNodePtr configuration::xmlFindTag(xmlNodePtr node, const String tag, int n)
{
   xmlNodePtr   element = NULL;

   while (element == NULL && node != NULL)
   {
      if (node->type == XML_ELEMENT_NODE && tag == (char *)node->name)
      {
         if (n == 0)
         {
            element = node;
         }
         n--;
      }
      node = node->next;
   }

   return element;
}

/*=========================================================================
**  NAME           : configuration::find_parameter
**  SYNOPSIS       :
**  PARAMETERS     :
**  RETURN VALUE   : The content of the parameter or an empty string
**
**  DESCRIPTION    : 
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Apr 13, 2007
**=========================================================================
*/

String configuration::find_parameter(const String section, const String parameter, int sec_n, int param_n)
{
   xmlNodePtr     root_node;
   xmlNodePtr     section_node;
   xmlNodePtr     param_node;

   String         param_value("");

   //  First, we try to find the parameter in the system-wide config

   if (system)
   {
      root_node    = xmlDocGetRootElement(system);
      section_node = xmlFindTag(root_node->children, section, sec_n);
      if (section_node != NULL)
      {
         param_node = xmlFindTag(section_node->children, parameter, param_n);
         if (param_node != NULL)
         {
            param_node = param_node->children;
         }
         if (param_node != NULL)
         {
            param_value = (char *)param_node->content;
         }
      }
   }

   //  If the parameter is also defined in the user config, it will override
   //  the system-wide value.

   if (user)
   {
      root_node    = xmlDocGetRootElement(user);
      section_node = xmlFindTag(root_node->children, section, sec_n);
      if (section_node != NULL)
      {
         param_node = xmlFindTag(section_node->children, parameter, param_n);
         if (param_node != NULL)
         {
            param_node = param_node->children;
         }
         if (param_node != NULL)
         {
            param_value = (char *)param_node->content;
         }
      }
   }

   return param_value;
}
